@@ -53,6 +53,7 @@ def predict_model(config: dict, fun_control: dict) -> Tuple[float, float]:
     """
     _L_in = fun_control["_L_in"]
     _L_out = fun_control["_L_out"]
+    _L_cond = fun_control["_L_cond"]
     _torchmetric = fun_control["_torchmetric"]
     if fun_control["enable_progress_bar"] is None:
         enable_progress_bar = False
@@ -74,7 +75,7 @@ def predict_model(config: dict, fun_control: dict) -> Tuple[float, float]:
     # TODO: Check if this is necessary:
     # dm.setup(stage="train")
     # Init model from datamodule's attributes
-    model = fun_control["core_model"](**config, _L_in=_L_in, _L_out=_L_out, _torchmetric=_torchmetric)
+    model = fun_control["core_model"](**config, _L_in=_L_in, _L_out=_L_out, _L_cond=_L_cond, _torchmetric=_torchmetric)
 
     trainer = L.Trainer(
         # Where to save models
